<?php
//Директоря с шаблонами
define ("TEMPLATES_DIR", "templates/{$config['template_name']}");
define ("TEMPLATES_DIR_ADM", ENGINE_DIR . "/admin/skin");

class template {

	protected $dir = ''; //Директория для шаблонов
	protected $result = array(); //Данные компиляции шаблонов
	protected $data = array(); //Данные для замены переменных
	protected $data_block = array(); //Данные для замены блоков
	protected $source = array(); //Исходные коды шаблонов
	protected $time_busy = 0; //Время занятое для компиляции
	protected $delete_tags = true; //Удалять неидентифицированные переменные из шаблона?
	protected $tag_start_delim = "{"; //Открывающий тэг переменной
	protected $tag_end_delim = "}"; //Закрывающий тэг переменной
	protected $block_start_delim_1 = "["; //Открывающий тэг открывающего блока
	protected $block_end_delim_1 = "]"; //Закрывающий тэг открывающего блока
	protected $block_start_delim_2 = "[/"; //Открывающий тэг закрывающего блока
	protected $block_end_delim_2 = "]"; //Закрывающий тэг закрывающего блока

	function __construct ($dir) {
		if (is_dir ($dir)) {
			$this->dir = $dir;
		} else {
			$this->trigger_error ('class: dirrectory ' . $dir . ' not found');
		}
	}

	//Открытие шаблона
	function open ($template, $name) {
		if (is_file ($this->dir . '/' . $template)) {
			$this->source[$name] = file_get_contents ($this->dir . '/' . $template);
			if (!isset ($this->data[$name])) $this->data[$name] = array();
			if (!isset ($this->data_block[$name])) $this->data_block[$name] = array();
		} else {
			$this->trigger_error ('open: template ' . $template . ' not found');
			$this->source[$name] = '';
		}
	}

	//Проверка существования перменной в шаблоне
	function isset_value ($name, $template) {
		if (isset($this->source[$template])) {
			if (strstr($this->source[$template], $tag_start_delim . $name . $tag_end_delim)) {
				return true;
			} else {
				return false;
			}
		} else {
			$this->trigger_error ('isset: template ' . $template . ' not found');
			return false;
		}
	}

	//Установка переменной. Если переменная не указана, то она вырезается из шаблона при компиляции
	function set ($name = '', $value = '', $template = '') {
		if ($name && $template) {
			$this->data[$template][$name] = $value;
		}
	}

	//Задание блока в указанном шаблоне
	function set_block ($name, $value, $template) {
		$this->data_block[$template][$name] = $value;
	}

	//Очистка значений для указанного шаблона
	function clear ($template) {
		$this->data[$template] = array ();
		$this->data_block[$template] = array ();
	}

	//Полная очистка значений, исходников, результатов
	function full_clear () {
		$this->data = array ();
		$this->data_block = array ();
		$this->result = array ();
		$this->source = array ();
	}

	function compile ($template, $return = false) {
		if (isset ($this->source[$template])) {

			$replace = isset ($this->data[$template]) ? $this->data[$template] : array();
			$block_replace = isset ($this->data_block[$template]) ? $this->data_block[$template] : array();

			$time = microtime ();

			$result = $this->source[$template];

            while (preg_match ('#\[(.*?)\].*?\[/\\1\]#s', $result)) {
                $result = preg_replace_callback ('#\[(.*?)\](.*?)\[/\\1\]#s', function ($match) use ($block_replace) {
                    return (isset ($block_replace[$match[1]])) ? $match[2] : '';
                }, $result);
            }

            $result = preg_replace_callback ('#\{(.*?)\}#s', function ($match) use ($replace) {
            	global $banners;

            	if (is_array ($banners) AND array_key_exists ($match[1], $banners)) {

            		return isset ($banners[$match[1]]) ? $banners[$match[1]] : ($this->delete_tags ? '' : "\{$match[1]\}");

            	} else {

            		return isset ($replace[$match[1]]) ? $replace[$match[1]] : ($this->delete_tags ? '' : "\{$match[1]\}");

            	}
            },  $result);

			/*preg_match ('#<script.+?>(.+?)</script>#s', $result, $script);
			if (isset ($script) AND !empty ($script[1])) {
                $result = preg_replace ("#<script.+?>.+?</script>#si", "<script>" . $script[1] . "</script>", $result);
            }*/

			list ($msec1, $sec1) = explode (' ', $time);
			list ($msec2, $sec2) = explode (' ', microtime ());
			$this->time_busy = $this->time_busy + (($msec2 + $sec2) - ($msec1 + $sec1));

			$this->result[$template] = $result;
			$this->clear ($template);

			if ($return) {
				return ($result);
			}

		} else {
			$this->trigger_error ('compile: template ' . $template . ' not open');
			$this->result[$template] = false;

			if ($return) {
				return false;
			}
		}
	}

	//Вывод ошибок
	function trigger_error ($error_msg, $error_type = E_USER_WARNING) {
		trigger_error ('Template error: ' . $error_msg, $error_type);
	}
}

?>